#
# This file is a portion of the Red Hat Update Agent
# Copyright (c) 1999, 2000 Red Hat, Inc.  Distributed under GPL
#
# $Id: config.py,v 1.29 2001/12/13 22:04:12 alikins Exp $
"""This module includes the Config and Up2date Config classes use
by the up2date agent to hold config info."""

import string, os

class Config:
    "class for handling global config options for the client"
    def __init__(self):
	self.dict = {}
	self.fileName = None

    def load(self):
	if self.fileName == None:
	    return
	baseName = string.split(self.fileName, "/")[-1:][0]
	baseName = string.split(baseName, ".")[0]

        if not os.access(self.fileName, os.R_OK):
            print "warning: can't access %s" % self.fileName
            return
        
        f = open(self.fileName, "r")

	key = ""
	comment = ""
	value = ""
	for line in f.readlines():
	    if string.split(line) == []:
		continue
	    if line[0] == "#":
		continue
	    try:
		(key, value) = string.split(line, "=", 2)
	    except:
		continue

	    key = string.lstrip(key)
	    key = string.rstrip(key)
	    value = string.lstrip(value)
	    value = string.rstrip(value)

	    cf = 0
	    pos = string.find(key, "[comment]")
	    if pos != -1:
		key = key[:pos]
		cf = 1

	    if cf:
		comment = value
	    else:
		# possibly split value into a list
		values = string.split(value, ";")
		if len(values) == 1:
		    try:
			value = int(value)
		    except:
			pass
		    self.dict[key] = (comment, value)
		elif values[0] == "":
		    self.dict[key] = (comment, [])
		else:
		    self.dict[key] = (comment, values[:-1])
		comment = ""

	f.close()


    def save(self):
	if self.fileName == None:
	    return
	baseName = string.split(self.fileName, "/")[-1:][0]
	baseName = string.split(baseName, ".")[0]
        
        f = open(self.fileName, "w")
        os.chmod(self.fileName, 0600)

	f.write("# Automatically generated Red Hat Update Agent config file, do not edit.\n")
	f.write("# Format: 1.0\n")
	f.write("")
	for key in self.dict.keys():
	    val = self.dict[key]
	    f.write("%s[comment]=%s\n" % (key, val[0]))
	    if type(val[1]) == type([]):
		f.write("%s=" % key)
		if len(val[1]) > 0:
		    for l in val[1]:
			f.write("%s;" % l)
		else:
		    f.write(";")
		f.write("\n")
	    else:
		f.write("%s=%s\n" % (key, val[1]))
	    f.write("\n")

	f.close()
	    
    def has_key(self, name):
        return self.dict.has_key(name)

    def keys(self):
	return self.dict.keys()

    def values(self):
	return self.dict.values()

    # return a dict of just param:value
    def readValues(self):
	tmpdict = {}
	for key in self.dict.keys():
	    tmpdict[key] =  self.dict[key][1]
	return tmpdict

    def readEntry(self, name):
	try:
	    val = self.dict[name]
	except:
	    val = (None, None)

	return val[1]


    def writeEntry(self, name, value):
	try:
	    val = self.dict[name]
	except:
	    val = (None, None)

	val = (val[0], value)
	self.dict[name] = val

    def hasEntry(self, name):
        return self.dict.has_key(name)

class RegisterPeekConfig(Config):
    "class to peek into the rhn_register configuration to see if we should use any of it's values"
    def __init__(self):
        Config.__init__(self)
        self.fileName = "/etc/sysconfig/rhn/rhn_register"
        
    def save(self):
        "just to be on the safe side..."
        pass


class Up2dateConfig(Config) :
    "derived from the Config class, with prepopulated default values"

    def __init__(self):
	Config.__init__(self)

	self.fileName = "/etc/sysconfig/rhn/up2date"
        self.dict['enableProxy'] = ("Use a HTTP Proxy", 0)
        self.dict['serverURL'] = ("Remote server URL",
                                  "https://www.rhns.redhat.com/XMLRPC")
        self.dict['noSSLServerURL'] = ("Remote server URL without SSL",
                                       "http://www.rhns.redhat.com/XMLRPC")

	self.dict['useNoSSLForPackages'] = ("Use the noSSLServerURL for package, package list, and header fetching",
					     0)
	self.dict['debug'] = ("Whether or not debugging is enabled", 0)
        self.dict['systemIdPath'] = ("Location of system id",
                                     "/etc/sysconfig/rhn/systemid")
	self.dict['adminAddress'] = ("List of e-mail addresses for update agent to communicate with when run in batch mode", ["root@localhost"])
	self.dict['storageDir'] = ("Where to store packages and other data when they are retrieved", "/var/spool/up2date")
	self.dict['pkgSkipList'] = ("A list of package names, optionally including wildcards, to skip", ["kernel*"])
	self.dict['removeSkipList'] = ("A list of package names, optionally including wildcards, that up2date will not remove", ["kernel*"])
	self.dict['fileSkipList'] = ("A list of file names, optionally including wildcards, to skip", [])
	self.dict['noReplaceConfig'] = ("When selected, no packages that would change configuration data are automatically installed", 1)
	self.dict['retrieveOnly'] = ("Retrieve packages only", 0)
        self.dict['retrieveSource'] = ("Retrieve source RPM along with binary package", 0)
	self.dict['keepAfterInstall'] = ("Keep packages on disk after installation", 0)
	self.dict['versionOverride'] = ("Override the automatically determined system version", "")
        self.dict['useGPG'] = ("Use GPG to verify package integrity", 1)
	self.dict['headerCacheSize'] = ("The maximum number of rpm headers to cache in ram", 40)
	self.dict['headerFetchCount'] = ("The maximimum number of rpm headers to fetch at once", 10)
	self.dict['forceInstall'] = ("Force package installation, ignoring package, file and config file skip list", 0)
        self.dict['httpProxy'] = ("HTTP proxy in host:port format, e.g. squid.redhat.com:3128", "")
        self.dict['proxyUser'] = ("The username for an authenticated proxy", "")
        self.dict['proxyPassword'] = ("The password to use for an authenticated proxy", "")
        self.dict['enableProxyAuth'] = ("To use an authenticated proxy or not", 0)
        self.dict['noBootLoader'] = ("To disable modification of the boot loader (lilo, silo, etc)", 0)	
        self.dict['networkRetries'] = ("Number of attempts to make at network connections before giving up", 5)
        self.dict['sslCACert'] = ("The location of the SSL CA certificate.", "/usr/share/rhn/RHNS-CA-CERT")
        
