#/usr/bin/python
# Client code for Update Agent
# Copyright (c) 1999-2001 Red Hat, Inc.  Distributed under GPL.
#
# Author: Preston Brown <pbrown@redhat.com>
#         Adrian Likins <alikins@redhat.com>
#
"""utility functions for up2date"""

import re
import os
import rpm
import string
import md5
from config import Up2dateConfig
cfg = Up2dateConfig()
cfg.load()

def getPackageSearchPath():
    dir_list = []
    dir_list.append(cfg.readEntry("storageDir"))

    dir_string = cfg.readEntry("packageDir")
    if dir_string: 
    	dir_list = dir_list + string.split(dir_string, ':')
    return dir_list

def pkgToString(pkg):
    return "%s-%s-%s" % (pkg[0], pkg[1], pkg[2])

# compare two RPM packages
def comparePackages(pkgLabel1, pkgLabel2):
    version1 = pkgLabel1[1]
    release1 = pkgLabel1[2]
    epoch1 = pkgLabel1[3]
    version2 = pkgLabel2[1]
    release2 = pkgLabel2[2]
    epoch2 = pkgLabel2[3]

    if epoch1 == "":
        epoch1 = None
    else:
        epoch1 = "%s" % epoch1
        
    if epoch2 == "":
            epoch2 = None
    else:
        epoch2 = "%s" % epoch2
        
    return rpm.labelCompare((epoch1, version1, release1),
                            (epoch2, version2, release2))


def md5sum(fileName):
    hash = md5.new()
    
    try:
        f = open(fileName, "r")
    except:
        return ""

    fData = f.read()
    hash.update(fData)
    del fData
    f.close()
    
    hex = string.hexdigits
    md5res = ""
    for c in hash.digest():
        i = ord(c)
        md5res = md5res + hex[(i >> 4) & 0xF] + hex[i & 0xf]

    return md5res

# return a glob for your particular architecture.
def archGlob():
    if re.search("i.86", os.uname()[4]):
        return "i?86"
    elif re.search("sparc", os.uname()[4]):
        return "sparc*"
    else:
        return os.uname()[4]




# small class for header caching pretending to be a dict
# in theory we could eventually make this smart and stuff
# (ie, mru, etc)
#
# implement it this way for now so we can make a max header
# cache size
#
# Need a miniHeaderCache, which caches all headers
# but only: name,version,release,arch,summary,description,size
# Otherwise the size computations are slow as dirt
#
class HeaderCache:
    def __init__(self):
        # how many headers to cache in ram
        if cfg.readEntry("headerCacheSize"):
            self.cache_size = cfg.readEntry("headerCacheSize")
        else:
            self.cache_size = 30
        self.__cache = {}
        self.__cacheLite = {}

    def set_cache_size(self, number_of_headers):
        self.cache_size = number_of_headers

    def __liteCopy(self, header):
        tmp = {}
        tmp['name'] = header['name']
        tmp['version'] = header['version']
        tmp['release'] = header['release']
        tmp['arch'] = header['arch']
        tmp['summary'] = header['summary']
        tmp['description'] = header['description']
        tmp['size'] = header['size']
        return tmp

    def __setitem__(self,item,value):
        if len(self.__cache) <= self.cache_size:
            self.__cache[item] = value
            self.__cacheLite[item] = self.__liteCopy(value)
        else:
            # okay, this is about as stupid of a cache as you can get
            # but if we hit the max cache size, this is as good as
            # any mechanism for freeing up space in the cache. This
            # would be a good place to put some smarts
            bar = self.__cache.keys()
            del self.__cache[bar[self.cache_size-1]]
            self.__cache[item] = value
            self.__cacheLite[item] = self.__liteCopy(value)

    def __getitem__(self, item):
        return self.__cache[item]
    

    def getLite(self, item):
        return self.__cacheLite[item]

    def __len__(self):
        return len(self.__cache)

    def keys(self):
        return self.__cache.keys()

    def values(self):
        return self.__cache.keys()

    def has_key(self, item,lite=None):
#        print "\n########\nhas_key called\n###########\n"
#        print "item: %s" % item
#        print self.__cache.keys()
        
        if lite:
            return self.__cacheLite.has_key(item)
        else:
            return self.__cache.has_key(item)

    def __delitem__(self, item):
        del self.__cache[item]

    def printLite(self):
        print self.__cacheLite
